function randomslits(varargin)
% ---
% --- function to create movies of scattering (speckle) pattern from an
% --- assembly of 200 random slits
% ---
% --- Input variable: "select" 
% ---         1: small random move away from starting position
% ---         2: small random move away from last position
% ---         3: completely new set of random slits
% ---
% --- This function creates a figure with 4 subplots:
% --- 1: Arrangement of random slits
% --- 2: Speckle pattern of 1)
% --- 3: Sum of speckle pattern
% --- 4: Zoom into sum of speckle pattern
% ---
% --- There is a pause after the figure is created for the first time
% --- in order to stretch the figure to its desired size before the first
% --- movie frame is stored. However, maximizing is probably too big!
% --- After the space bar is hit, a movie of 200 frames is created.
% ---

if ( nargin >= 1 )
    select = varargin{1};
else
    select = 2;
end

% ---slit scattering from slits of equal area but different shape
bitsperpixel = 14;
cutoff       = (2^bitsperpixel - 1);

% --- many small quadratic slits on horizontal row (6x 16x16 pixels)
n = 200;
r = rand(n,2);

row = zeros(n,1);
col = zeros(n,1);

for i = 1 : n
   row(i) = floor(512*r(i,1))+1023-512/2;
   col(i) = floor(512*r(i,2))+1023-512/2;
end

repeats = 200;
mov(repeats) = struct('cdata',[],'colormap',[]);
c = zeros(2048);
for j = 1 : repeats

    rj = rand(n,2);
    if ( j == 1 || select == 1 || select == 3 )
        rowj = zeros(n,1);
        colj = zeros(n,1);
    end
    for i = 1 : n
        % --- small random move from the same starting position
        if ( select == 1 )
            rowj(i) = row(i)+ round((500000 - floor(1000000*rj(i,1)))/250000);
            colj(i) = col(i)+ round((500000 - floor(1000000*rj(i,2)))/250000);
        end
        % --- small random move from last position
        if ( select == 2 )
            if (j > 1 && i == 1 )
                row = rowj;
                col = colj;
            end
            rowj(i) = row(i) + (500000 - floor(1000000*rj(i,1))) / 250000;
            colj(i) = col(i) + (500000 - floor(1000000*rj(i,2))) / 250000;
        end
        % --- completely new set of random slits
        if ( select == 3 )
            rowj(i) = floor(512*rj(i,1))+1023-512/2;
            colj(i) = floor(512*rj(i,2))+1023-512/2;
        end
    end
    
    a = zeros(2048);
    for i = 1 : n
        a(round(rowj(i))-2:round(rowj(i))+2, round(colj(i))-2:round(colj(i))+2) ...
            = a(round(rowj(i))-2:round(rowj(i))+2, round(colj(i))-2:round(colj(i))+2) + 1 ;
    end
    
    b = fftshift(fft2(a).*conj(fft2(a)));
    bmax = max(b(:));
    b(b<=bmax/cutoff)=bmax/cutoff;

    c = c + b;
    cmax = max(c(:));
    d = c ;
    d(d<=cmax/(j*cutoff))=cmax/(j*cutoff);
    
    % ---
    figure(1)
    subplot(2,2,1); imagesc(a(641:1408,641:1408)); axis square;
    subplot(2,2,2); imagesc(log10(b(769:1280,769:1280))); axis square;
    subplot(2,2,3); imagesc(log10(d(769:1280,769:1280))); axis square;
    subplot(2,2,4); imagesc(log10(d(945:1104,945:1104))); axis square;
    colormap(hot(255));

    if (j == 1)
        pause
    end
    
    figure(1)    
    mov(j) = getframe(gcf);
    
end

if ( select == 1 )
    filename  = '\\win.desy.de\home\sprung\My Documents\MATLAB\Utilities\dummy\speckle_01.mp4';
end
if ( select == 2 )
    filename  = '\\win.desy.de\home\sprung\My Documents\MATLAB\Utilities\dummy\speckle_02.mp4';
end
if ( select == 3 )
    filename  = '\\win.desy.de\home\sprung\My Documents\MATLAB\Utilities\dummy\speckle_03.mp4';
end
writerObj = VideoWriter(filename,'MPEG-4');
writerObj.FrameRate = 10;
open(writerObj);
writeVideo(writerObj,mov);
close(writerObj);

% --- clear some variables
clear bitsperpixel cutoff n r row col

% ---
% EOF
